package com.cz.uhk.bakalarka.app.controller;

import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import com.cz.uhk.bakalarka.app.service.InputMessageValidationService;
import com.cz.uhk.bakalarka.app.service.JsonParserService;
import com.cz.uhk.bakalarka.app.service.MessageService;
import com.cz.uhk.bakalarka.app.service.XmlParserService;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.*;
import com.cz.uhk.bakalarka.app.service.interfaces.InputMessageValidationServiceInf;
import com.cz.uhk.bakalarka.app.service.interfaces.MessageServiceInf;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

/**
 * Controller that handle request to API.
 */
@RestController
@RequestMapping("/api")
@Log4j2
public class MessageController {

    private final MessageServiceInf messageService;
    private final InputMessageValidationServiceInf messageValidationService;
    private final JsonParserService jsonParserService;
    private final XmlParserService xmlParserService;

    @Autowired
    public MessageController(MessageService messageService, InputMessageValidationService inputMessageValidationService, JsonParserService jsonParserService, XmlParserService xmlParserService) {
        this.messageService = messageService;
        this.messageValidationService = inputMessageValidationService;
        this.jsonParserService = jsonParserService;
        this.xmlParserService = xmlParserService;
    }


    /**
     * Control processing of request to save file into repository. Validation, parsing file to metadata and send
     * payload and metadata to service layer.
     * @param file file from request to save
     * @return information regarding the course of processing
     * @throws Exception throws becouse sseEmitter
     */
    @PostMapping(value = "/create")
    public ResponseEntity<SseEmitter> create(@RequestParam("file") MultipartFile file) throws Exception {
        SseEmitter sseEmitter = new SseEmitter();
        try {
            String name = file.getOriginalFilename();
            log.info("File " + name + " is received on the api endpoint.");
            file.getInputStream().close();
            InputStream payload = file.getInputStream();
            String[] validationResponse = messageValidationService.validateCreate(file);
            log.info("File " + validationResponse[1] + " is successfully validated.");
            Map<String, String> metaDataMap = new HashMap<>();

            if (validationResponse[0].equals(AllowedFileFormat.JSON.name())) {
                metaDataMap = jsonParserService.parseToMap(file);

            } else if (validationResponse[0].equals(AllowedFileFormat.XML.name())) {
                metaDataMap = xmlParserService.parseToMap(file);
            }

            metaDataMap.put(MetaDataTypes.CODE.name(), validationResponse[1]);
            log.info("File is successfully parsed to the map with code: " + validationResponse[1]);
            MetaData metaData = new MetaData(metaDataMap);

            sseEmitter.send(SseEmitter.event().name("validation success").data("File was successfully validated"));
            messageService.create(payload, metaData, sseEmitter);

        } catch (EmptyFileException e) {
            sseEmitter.send(SseEmitter.event().name("File exception").data("File is empty"));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.BAD_REQUEST);

        } catch (LargeFileException e) {
            sseEmitter.send(SseEmitter.event().name("File exception").data("The file is larger than is allowed"));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.BAD_REQUEST);

        } catch (MissingRequiredMetaDataException e) {
            sseEmitter.send(SseEmitter.event().name("File exception").data(e.getMessage()));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.BAD_REQUEST);

        } catch (ParsingFileContentSyntaxException e) {
            sseEmitter.send(SseEmitter.event().name("File exception").data("In the file is syntax error"));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.BAD_REQUEST);

        } catch (UnsupportedFileFormatException e) {
            sseEmitter.send(SseEmitter.event().name("File exception").data("This file format server cannot processed"));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.BAD_REQUEST);

        } catch (Exception e) {
            log.error("There is problem. " + e.getMessage() + "  " + e.getCause());
            sseEmitter.send(SseEmitter.event().name("Transfer problem").data("Problem with file transfer to api"));
            sseEmitter.complete();
            return new ResponseEntity<>(sseEmitter, HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(sseEmitter, HttpStatus.ACCEPTED);
    }


    /**
     * Get code according to code. As part of processing code correctness.
     * @param code code of object
     * @return Metadata of object
     */
    @GetMapping("/getByCode")
    public ResponseEntity<String> getByCode(@RequestParam("code") String code) {
        try {
            log.info("Request is received on the api endpoint with code: " + code);
            messageValidationService.validateCodeCorrectness(code);
            log.info("Code passed pattern validation with code: " + code);
            return ResponseEntity.ok(messageService.getByCode(code).toString());

        } catch (PayloadDaoErrorResponseException e) {
            return new ResponseEntity<>("File with code: " + code + " doesn´t exist or bucket doesn´t exist", HttpStatus.NOT_FOUND);

        } catch (PayloadDaoException e) {
            return new ResponseEntity<>("There is internal error, try again later", HttpStatus.INTERNAL_SERVER_ERROR);

        } catch (BadCodeFormatException e) {
            return new ResponseEntity<>("The selected code have bad pattern, try again", HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * Get payload of the object according to code. As part of processing code correctness.
     * @param code code of object
     * @return payload of object
     */
    @GetMapping("/getDataByCode")
    public ResponseEntity<String> getDataByCode(@RequestParam("code") String code) {
        try {
            log.info("Request is received on the api endpoint with code: " + code);
            messageValidationService.validateCodeCorrectness(code);
            log.info("Code passed pattern validation with code: " + code);
            InputStream payload = messageService.getDataByCode(code);
            byte[] arrayPayload = payload.readAllBytes();
            payload.close();
            String payloadToOutput = new String(arrayPayload, StandardCharsets.UTF_8);
            return ResponseEntity.ok(payloadToOutput);

        } catch (PayloadDaoErrorResponseException e) {
            return new ResponseEntity<>("File with code: " + code + " doesn´t exist or bucket doesn´t exist", HttpStatus.NOT_FOUND);

        } catch (PayloadDaoException e) {
            return new ResponseEntity<>("There is internal error, try again later", HttpStatus.INTERNAL_SERVER_ERROR);

        } catch (BadCodeFormatException e) {
            return new ResponseEntity<>("The selected code have bad pattern, try again", HttpStatus.BAD_REQUEST);

        } catch (Exception e) {
            return new ResponseEntity<>("There is internal IO error", HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * Control delete object according to code. In part of processing is code correctness.
     * @param code
     * @return message about success or error
     */
    @DeleteMapping("/delete")
    public ResponseEntity<String> delete(@RequestParam("code") String code) {
        try {
            log.info("Request is received on the api endpoint with code: " + code);
            messageValidationService.validateCodeCorrectness(code);
            log.info("Code passed pattern validation with code: " + code);
            messageService.delete(code);
            return new ResponseEntity<>("The file with code: " + code + " was deleted", HttpStatus.OK);

        } catch (PayloadDaoErrorResponseException e) {
            return new ResponseEntity<>("There is error response from repository. It may be non existent bucket", HttpStatus.INTERNAL_SERVER_ERROR);

        } catch (PayloadDaoException e) {
            return new ResponseEntity<>("There is internal error, try again later", HttpStatus.INTERNAL_SERVER_ERROR);

        } catch (BadCodeFormatException e) {
            return new ResponseEntity<>("The selected code have bad pattern, try again", HttpStatus.BAD_REQUEST);
        }
    }
}
