package com.cz.uhk.bakalarka.app.dao;

import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadErrorResponseException;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadException;
import com.cz.uhk.bakalarka.app.dao.interfaces.MessagePayloadDao;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import io.minio.*;
import io.minio.errors.ErrorResponseException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Repository;
import java.io.InputStream;
import java.util.Map;

/**
 * Repository that working with MinIO.
 */
@Repository
@Log4j2
public class MessagePayloadMinIoDao implements MessagePayloadDao<MetaData> {
    private final MinioClient minioClient;
    private final String bucket;

    public MessagePayloadMinIoDao(MinioClient minioClient, String defaultBucket) {
        this.minioClient = minioClient;
        this.bucket = defaultBucket;
    }

    @Autowired
    public MessagePayloadMinIoDao(@Value("${api.minio.url}") String url, @Value("${api.minio.password}") String password, @Value("${api.minio.username}") String username, @Value("${api.minio.default_bucket}") String defaultBucket) {
        log.info("Start initialize with " + url);
        minioClient = MinioClient.builder()
                .endpoint(url)
                .credentials(username, password)
                .build();
        this.bucket = defaultBucket;
        log.info("MessagePayloadMinIoDao initialized");
    }


    /**
     * Method saves object and metadata to MinIO object storage.
     * @param payload Data that is saved.
     * @param metadata Metadata from the saved file.
     * @return metadata that was saved
     * @throws MinioPayloadException throws because ErrorResponseException or other exceptions
     */
    @Override
    public MetaData create(InputStream payload, MetaData metadata) throws MinioPayloadException {
        try {
            Map<String, String> metaDataMap = metadata.getMetaDataMap();
            log.info("File is saving with code: " + metadata.getCode());

            minioClient.putObject(PutObjectArgs.builder()
                    .bucket(bucket)
                    .object(metadata.getCode())
                    .stream(payload, -1, 10485760)
                    .contentType(metadata.getContentType())
                    .userMetadata(metaDataMap).build());
            return getByCode(metadata.getCode());

        } catch (ErrorResponseException e) {
            log.error("There is probably problem with no existent bucket or over sized metadata.");
            throw new MinioPayloadErrorResponseException(e.getMessage(), e, metadata);

        } catch (Exception e) {
            log.error("There occurs problem when app trying to save the file. File code: " + metadata.getCode());
            throw new MinioPayloadException(e.getMessage(), e, metadata);
        }
    }

    /**
     * Method deleting file from MinIO object storage.
     * @param code File is saves with this code. Code identify each file in the repository.  In this case
     *             code identify object, which must be deleted.
     * @throws MinioPayloadException throws because ErrorResponseException or other exceptions
     */
    @Override
    public void delete(String code) throws MinioPayloadException {
        try {
            log.info("Object with code " + code + " will be removed");
            minioClient.removeObject(RemoveObjectArgs.builder()
                    .bucket(bucket)
                    .object(code)
                    .build());
        } catch (ErrorResponseException e) {
            log.error("There is probably problem with no existent bucket. Code is " + code);
            throw new MinioPayloadErrorResponseException(e.getMessage(), e);

        } catch (Exception e) {
            log.error("There occurs problem when app trying to remove the file. Code is " + code);
            throw new MinioPayloadException(e.getMessage(), e);
        }
    }

    /**
     * Method that return payload of object based on code.
     * @param code  File is saves with this code. Code identify each file in the repository. In this case
     *              code identify object, which´s data must be returned.
     * @return inputStream of the payload
     * @throws MinioPayloadException throws because ErrorResponseException or other exceptions
     */
    @Override
    public InputStream getDataByCode(String code) throws MinioPayloadException {
        try {
            log.info("Data from the object with code " + code + "will be on the way.");
            return minioClient.getObject(GetObjectArgs.builder()
                    .bucket(bucket)
                    .object(code)
                    .build());

        } catch (ErrorResponseException e) {
            log.error("There is probably problem with no existent bucket or object. Finding object code is " + code);
            throw new MinioPayloadErrorResponseException(e.getMessage(), e);

        } catch (Exception e) {
            log.error("There occurs problem when app trying to get data from the file. Finding object code is " + code);
            throw new MinioPayloadException(e.getMessage(), e);
        }
    }

    /**
     * Method return metadata of the object based on code.
     * @param code File is saves with this code. Code identify each file in the repository. In this case
     *             code identify object, which´s metadata must be returned.
     * @return metadata of the object.
     * @throws MinioPayloadException throws because ErrorResponseException or other exceptions
     */
    @Override
    public MetaData getByCode(String code) throws MinioPayloadException {
        try {
            log.info("Metadata from the file for the " + code + " code will be on the way");
            StatObjectResponse metaDataResponse = minioClient.statObject(StatObjectArgs.builder()
                    .bucket(bucket)
                    .object(code)
                    .build());

            Map<String, String> mapMetaData = metaDataResponse.userMetadata();

            return new MetaData(
                    mapMetaData.get(MetaDataTypes.FILE_ID.name().toLowerCase()),
                    metaDataResponse.object(),
                    mapMetaData.get(MetaDataTypes.FILE_NAME.name().toLowerCase()),
                    metaDataResponse.contentType(),
                    mapMetaData.get(MetaDataTypes.VERSION_NAME.name().toLowerCase()),
                    Boolean.parseBoolean(mapMetaData.get(MetaDataTypes.ARCHIVED.name().toLowerCase())),
                    Boolean.parseBoolean(mapMetaData.get(MetaDataTypes.DELETED.name().toLowerCase())),
                    mapMetaData.get(MetaDataTypes.SENDING_ACCOUNT_ID.name().toLowerCase()),
                    mapMetaData.get(MetaDataTypes.RECEIVING_ACCOUNT_ID.name().toLowerCase()));



        } catch (ErrorResponseException e) {
            log.error("There is probably problem with no existent bucket or object. Finding object code is " + code);
            throw new MinioPayloadErrorResponseException(e.getMessage(), e);

        } catch (Exception e) {
            log.error("There occur problem when minio trying to get metadata from the file. Finding object code is " + code);
            throw new MinioPayloadException(e.getMessage(), e);
        }
    }


    /**
     * Method that return boolean, true if object exists and false if object is not exists.
     * @param code File is saves with this code. Code identify each file in the repository. In this case
     *             code identify object whose existence need to be decided.
     * @return boolean of result
     * @throws MinioPayloadException throws because exceptions
     */
    @Override
    public boolean exists(String code) throws MinioPayloadException {
        try {
            getByCode(code);
            log.info("Object with code " + code + " exist");
            return true;

        } catch (MinioPayloadErrorResponseException e) {
            log.info("Object with code " + code + " doesn´t exist");
            return false;

        } catch (Exception e) {
            log.error("There occur problem when minio trying to find corresponding object with code: " + code);
            throw new MinioPayloadException(e.getMessage(), e);

        }
    }
}
