package com.cz.uhk.bakalarka.app.model;


import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import lombok.Getter;
import lombok.Setter;

import java.util.HashMap;
import java.util.Map;

/**
 * Class that represents file metadata
 */
@Getter
@Setter
public class MetaData {
    private String fileId;
    private String code;
    private String fileName;
    private String contentType;
    private String versionName;
    private Boolean archived = false;
    private Boolean deleted = false;
    private String sendingAccountId;
    private String receivingAccountId;

    private Map<String, String> metaDataMap;

    public MetaData() {

    }

    /**
     * Create object of MetaData.
     * @param fileName name of the file
     * @param contentType content type of the file
     * @param code code, that is generated to identify the object
     */
    public MetaData(String fileName, String contentType, String code) {
        this.fileName = fileName;
        this.contentType = contentType;
        this.code = code;
        metaDataMap = generateMap();
    }

    /**
     * Create object of MetaData.
     * @param fileId messageId in the file.
     * @param code code of the file.
     * @param fileName name of the file.
     * @param contentType content type of the file.
     * @param versionName number of certificates in file.
     * @param archived if the file is archived.
     * @param deleted if the file is deleted.
     * @param sendingAccountId id of sending account.
     * @param receivingAccountId id of receiving account.
     */
    public MetaData(String fileId, String code, String fileName, String contentType,  String versionName, boolean archived, boolean deleted, String sendingAccountId, String receivingAccountId) {
        this.fileId = fileId;
        this.code = code;
        this.fileName = fileName;
        this.contentType = contentType;
        this.versionName = versionName;
        this.archived = archived;
        this.deleted = deleted;
        this.sendingAccountId = sendingAccountId;
        this.receivingAccountId = receivingAccountId;
        metaDataMap = generateMap();
    }

    /**
     * Create object of MetaData.
     * @param metaDataMap map, that contains metadata attributes.
     */
    public MetaData(Map<String, String> metaDataMap) {
        this.metaDataMap = metaDataMap;

         this.code = metaDataMap.get(MetaDataTypes.CODE.name());
         this.fileId = metaDataMap.get(MetaDataTypes.FILE_ID.name());
         this.fileName = metaDataMap.get(MetaDataTypes.FILE_NAME.name());
         this.contentType = metaDataMap.get(MetaDataTypes.CONTENT_TYPE.name());
         this.versionName = metaDataMap.get(MetaDataTypes.VERSION_NAME.name());
         this.archived = Boolean.valueOf(metaDataMap.get(MetaDataTypes.ARCHIVED.name()));
         this.deleted = Boolean.valueOf(metaDataMap.get(MetaDataTypes.DELETED.name()));
         this.sendingAccountId = metaDataMap.get(MetaDataTypes.SENDING_ACCOUNT_ID.name());
         this.receivingAccountId = metaDataMap.get(MetaDataTypes.RECEIVING_ACCOUNT_ID.name());
    }

    /**
     *It generates map of the Metadata attributes
     * @return
     */
    private Map<String, String> generateMap() {
        Map<String, String> metaDataMap = new HashMap<>();
        metaDataMap.put(MetaDataTypes.FILE_ID.name(), fileId);
        metaDataMap.put(MetaDataTypes.CODE.name(), code);
        metaDataMap.put(MetaDataTypes.CONTENT_TYPE.name(), contentType);
        metaDataMap.put(MetaDataTypes.FILE_NAME.name(), fileName);
        metaDataMap.put(MetaDataTypes.VERSION_NAME.name(), versionName);
        metaDataMap.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(archived));
        metaDataMap.put(MetaDataTypes.DELETED.name(), Boolean.toString(deleted));
        metaDataMap.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), sendingAccountId);
        metaDataMap.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), receivingAccountId);
        return metaDataMap;
    }

    @Override
    public String toString() {
        return "MetaData \n" +
                "   fileId= " + fileId + '\n' +
                "   code= " + code + '\n' +
                "   fileName= " + fileName + '\n' +
                "   contentType= " + contentType + '\n' +
                "   versionName= " + versionName + '\n' +
                "   archived= " + archived + "\n" +
                "   deleted= " + deleted + "\n" +
                "   receivingAccountId= " + receivingAccountId + "\n" +
                "   sendingAccountId= " + sendingAccountId + "\n";
    }
}
