package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.dao.MessagePayloadMinIoDao;
import com.cz.uhk.bakalarka.app.dao.interfaces.MessagePayloadDao;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.BadCodeFormatException;
import com.cz.uhk.bakalarka.app.service.exception.EmptyFileException;
import com.cz.uhk.bakalarka.app.service.exception.LargeFileException;
import com.cz.uhk.bakalarka.app.service.exception.UnsupportedFileFormatException;
import com.cz.uhk.bakalarka.app.service.interfaces.InputMessageValidationServiceInf;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import java.util.*;
import java.util.regex.Pattern;

/**
 * Class that is responsible to validate input message.
 */
@Service
@Log4j2
public class InputMessageValidationService implements InputMessageValidationServiceInf {

    private final MessagePayloadDao<MetaData> messagePayloadDao;
    private final AllowedFileFormat[] allowedFileFormat;
    private int maxSize;


    /**
     *
     * @param messagePayloadDao Dao, that is connected to MinIO.
     * @param size max size of the file, that is allowed.
     */
    @Autowired
    public InputMessageValidationService(MessagePayloadMinIoDao messagePayloadDao, @Value("${app.file.max_size}") int size) {
        this.messagePayloadDao = messagePayloadDao;
        allowedFileFormat = AllowedFileFormat.values();
        maxSize = size;
      }

    /**
     * Method, that validate input file. It is responsible to validate max size and allowed file format. If there
     * occurs file that is larger or with unsupported file format, exception is thrown. After success validation
     * method generate UUID code for further processing.
     * @param multipartFile file selected to the validation
     * @return Return array with two indexes. Index 0 contains format of the file, index 1 contains generated code.
     * @throws LargeFileException exception that is thrown if file is larger than is allowed.
     * @throws UnsupportedFileFormatException exception that is thrown if file has unsupported file format.
     * @throws EmptyFileException exception that is thrown if file is empty.
     */
    @Override
    public String[] validateCreate(MultipartFile multipartFile) throws LargeFileException, UnsupportedFileFormatException, EmptyFileException {

        if (multipartFile.getSize() > maxSize) {
            log.error("The file " + multipartFile.getOriginalFilename() + " is larger than is allowed");
            log.info("App ended process file  " + multipartFile.getOriginalFilename());
            throw new LargeFileException("The File is larger than is allowed");

        } else if (multipartFile.isEmpty()){
            log.error("The file content is empty in " + multipartFile.getOriginalFilename());
            log.info("App ended process file  " + multipartFile.getOriginalFilename());
            throw new EmptyFileException("File content is empty");
        }

        for (AllowedFileFormat format : allowedFileFormat) {
            String type = "application/" + format.name().toLowerCase();
            String contentType = multipartFile.getContentType();
            if (contentType != null) {
                if (contentType.contains(type)) {
                    boolean exists = true;

                    do {
                        try {
                            UUID uuid = UUID.randomUUID();

                            String code = uuid.toString();
                            exists = messagePayloadDao.exists(uuid.toString());

                            if (exists) {
                                log.error("Code for new object exists.");
                            } else {
                                log.info("New code for " + multipartFile.getOriginalFilename() + " accepted:" + code);
                                String[] response = new String[2];
                                response[0] = format.name();
                                response[1] = code;
                                return response;
                            }

                        } catch (Exception e) {
                            log.error(e.getMessage(), e.getCause());
                        }
                    } while (exists);
                }
            }
        }
        log.error("The file " + multipartFile.getOriginalFilename() + " cannot be processed, because bad file format");
        log.info("App ended process file " + multipartFile.getOriginalFilename());
        throw new UnsupportedFileFormatException("The file format server cannot be processed");
    }


    /**
     * Method that validate code correctness with regex pattern.
     * @param code input code to validate.
     * @return true if validation is successful, false if there is error.
     * @throws BadCodeFormatException exception that is thrown if code is in bad pattern.
     */
    @Override
    public boolean validateCodeCorrectness(String code) throws BadCodeFormatException {
        Pattern regEx = Pattern.compile("^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$");
        if (!regEx.matcher(code).matches()){
            log.error("The code have bad format, pattern is different than request code " + code);
            log.info("App ended process request with code: " + code);
            throw new BadCodeFormatException("The code is not compatible with pattern");
        } else {
            return true;
        }
    }

}
