package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.MissingRequiredMetaDataException;
import com.cz.uhk.bakalarka.app.service.exception.ParsingFileContentSyntaxException;
import com.cz.uhk.bakalarka.app.service.interfaces.ParserServiceInf;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonSyntaxException;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

/**
 * Class responsible to parse input file in JSON format into metadata.
 */
@Service
@Log4j2
public class JsonParserService implements ParserServiceInf {

    public JsonParserService() {
    }

    /**
     * Method parsing input file and generating map with all attributes. Attributes deleted and archived are
     * default set on false.
     * @param multipartFile File assigned to be parsed.
     * @return Map of the Metadata attributes.
     * @throws MissingRequiredMetaDataException exception that is thrown if in the file missing attribute.
     * @throws ParsingFileContentSyntaxException exception that is thrown if file content have bad syntax.
     */
    @Override
    public Map<String, String> parseToMap(MultipartFile multipartFile) throws MissingRequiredMetaDataException, ParsingFileContentSyntaxException {
        Map<String, String> map = new HashMap<>();
        String actualMetaData = "";
        try {
            InputStream inputStream = multipartFile.getInputStream();
            String content = new String(inputStream.readAllBytes(), StandardCharsets.UTF_8);

            Gson gson = new Gson();
            JsonObject json = gson.fromJson(content, JsonObject.class);

            JsonObject body = json.get("body").getAsJsonObject();
            JsonObject header = json.get("header").getAsJsonObject();

            actualMetaData = MetaDataTypes.FILE_NAME.name();
            String name = multipartFile.getOriginalFilename();
            map.put(actualMetaData, name);

            actualMetaData = MetaDataTypes.CONTENT_TYPE.name();
            String contentType = multipartFile.getContentType();
            map.put(actualMetaData, contentType);

            actualMetaData = "sendingAccountID";
            String sendingAccountId = body.getAsJsonObject(actualMetaData).get("value").getAsString();
            map.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), sendingAccountId);

            actualMetaData = "receivingAccountID";
            String receivingAccountId = body.getAsJsonObject(actualMetaData).get("value").getAsString();
            map.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), receivingAccountId);

            actualMetaData = "numberOfCertificates";
            String version = body.get(actualMetaData).getAsString();
            map.put(MetaDataTypes.VERSION_NAME.name(), version);

            actualMetaData = "messageID";
            String fileId = header.get(actualMetaData).getAsString();
            map.put(MetaDataTypes.FILE_ID.name(), fileId);

            map.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
            map.put(MetaDataTypes.DELETED.name(), Boolean.toString(false));

        } catch (NullPointerException e) {
            log.error("Required attribute missing in json file. The missing attribute is " + actualMetaData + "it must be repaired to continue");
            log.info("App ended process file " + multipartFile.getOriginalFilename());
            throw new MissingRequiredMetaDataException(actualMetaData);

        } catch (JsonSyntaxException e){
            log.error("There is problem with JSON structure.");
            log.info("App ended process file " + multipartFile.getOriginalFilename());
            throw new ParsingFileContentSyntaxException(AllowedFileFormat.JSON);

        } catch (Exception e){
            log.error(e.getMessage(),e.getCause());
        }

        return map;
    }


}
