package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.dao.MessagePayloadMinIoDao;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadErrorResponseException;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadException;
import com.cz.uhk.bakalarka.app.dao.interfaces.MessagePayloadDao;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.service.exception.PayloadDaoErrorResponseException;
import com.cz.uhk.bakalarka.app.service.exception.PayloadDaoException;
import com.cz.uhk.bakalarka.app.service.interfaces.MessageServiceInf;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.InputStream;

/**
* Service layer that transfer payload and Metadata from controller to DAO.
*/
@Service
@Log4j2
public class MessageService implements MessageServiceInf {

    private final MessagePayloadDao<MetaData> messageDao;

    @Autowired
    public MessageService(MessagePayloadMinIoDao messagePayloadDao) {
        this.messageDao = messagePayloadDao;
    }

    /**
     * @param payload payload from file.
     * @param metaData metadata of the file.
     * @param sseEmitter sseEmitter that is used to send information about processing.
     * @throws Exception All exception that can be thrown.
     */
    @Override
    @Async()
    public void create(InputStream payload, MetaData metaData, SseEmitter sseEmitter) throws Exception {

        try {
            messageDao.create(payload, metaData);
            log.info("object with code " + metaData.getCode() + " was succesfully uploaded");
            sseEmitter.send(SseEmitter.event().name("success upload").data("File was successfully uploaded"));
            sseEmitter.send(SseEmitter.event().name("code").data(metaData.getCode()));
            sseEmitter.complete();

        } catch (MinioPayloadErrorResponseException e) {
            log.error("File with code: " + metaData.getCode() + "  " + e.getMessage(), e.getCause());
            sseEmitter.send(SseEmitter.event().name("upload unsuccessful").data("There is error response from repository during uploading, file wasn´t upload. With file name " + e.getMetaData().getFileName() + ", try again later. Also it may be non-existent bucket").build());
            sseEmitter.completeWithError(e);

        } catch (MinioPayloadException e) {
            log.error("File with code: " + metaData.getCode() + "  " + e.getMessage(), e.getCause());
            sseEmitter.completeWithError(e);
            sseEmitter.send(SseEmitter.event().name("upload unsuccessful").data("There is internal error, try again later").build());
            sseEmitter.completeWithError(e);

        } catch (Exception e) {
            sseEmitter.send(SseEmitter.event().name("upload unsuccessful").data("There is internal error, try again later").build());
            sseEmitter.completeWithError(e);
            log.error("File with code: " + metaData.getCode() + "  " + e.getMessage(), e.getCause());
        }
    }


    /**
     * @param code code of the file, that is selected to delete.
     * @throws PayloadDaoException Exception that is thrown if is thrown MinioPayloadException.
     */
    @Override
    public void delete(String code) throws PayloadDaoException {
        try {
            messageDao.delete(code);
            log.info("The file with code " + code + " was successfully deleted");

        } catch (MinioPayloadErrorResponseException e) {
            log.error("Delete with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoErrorResponseException(e.getMessage(), e);

        } catch (MinioPayloadException e) {
            log.error("Delete with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoException(e.getMessage(), e);

        }
    }

    /**
     * @param code code of the file, which payload must be return.
     * @return InputStream of the payload.
     * @throws PayloadDaoException Exception that is thrown if is thrown MinioPayloadException.
     */
    @Override
    public InputStream getDataByCode(String code) throws PayloadDaoException {
        try {
            InputStream payload = messageDao.getDataByCode(code);
            log.info("The data of the file with code " + code + " was successfully returned");
            return payload;

        } catch (MinioPayloadErrorResponseException e) {
            log.error("Getting data with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoErrorResponseException(e.getMessage(), e);

        } catch (MinioPayloadException e) {
            log.error("Getting data with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoException(e.getMessage(), e);

        }
    }

    /**
     * @param code code of the file, which Metadata must be return.
     * @return Metadata of the requested file.
     * @throws PayloadDaoException payloadDaoException that is thrown
     */
    @Override
    public MetaData getByCode(String code) throws PayloadDaoException {
        try {
            MetaData metaData = messageDao.getByCode(code);
            log.info("The data of the file with code " + code + " was successfully returned");
            return metaData;

        } catch (MinioPayloadErrorResponseException e) {
            log.error("Getting metadata with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoErrorResponseException(e.getMessage(), e);

        } catch (MinioPayloadException e) {
            log.error("Getting metadata with code: " + code + " " + e.getMessage(), e.getCause());
            throw new PayloadDaoException(e.getMessage(), e);

        }
    }
}
