package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.MissingRequiredMetaDataException;
import com.cz.uhk.bakalarka.app.service.exception.ParsingFileContentSyntaxException;
import com.cz.uhk.bakalarka.app.service.interfaces.ParserServiceInf;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.w3c.dom.Document;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * Class responsible to parse input file in XML format into metadata.
 */
@Service
@Log4j2
public class XmlParserService implements ParserServiceInf {

    /**
     * Method parsing input file and generating map with all attributes.
     * @param multipartFile File assigned to be parsed.
     * @return Map of the Metadata attributes.
     * @throws MissingRequiredMetaDataException exception that is thrown if in the file missing attribute.
     * @throws ParsingFileContentSyntaxException exception that is thrown if file content have bad syntax.
     */
    @Override
    public Map<String, String> parseToMap(MultipartFile multipartFile) throws MissingRequiredMetaDataException, ParsingFileContentSyntaxException {
        Map<String,String> map = new HashMap<>();
        String actualMetaData = "";
        try {
            InputStream content = multipartFile.getInputStream();
            DocumentBuilder dbuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document document = dbuilder.parse(content);
            document.getDocumentElement().normalize();



            //parsovani jednotlivych elementu
            actualMetaData = MetaDataTypes.FILE_NAME.name();
            String name = multipartFile.getOriginalFilename();
            map.put(actualMetaData, name);

            actualMetaData = MetaDataTypes.CONTENT_TYPE.name();
            String contentType = multipartFile.getContentType();
            map.put(actualMetaData, contentType);

            actualMetaData = "r:SendingAccountID";
            String sendingAccountId = document.getElementsByTagName(actualMetaData).item(0).getFirstChild().getTextContent();
            map.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), sendingAccountId);

            actualMetaData = "r:ReceivingAccountID";
            String receivingAccountID = document.getElementsByTagName(actualMetaData).item(0).getFirstChild().getTextContent();
            map.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), receivingAccountID);

            actualMetaData = "r:NumberOfCertificates";
            String version = document.getElementsByTagName(actualMetaData).item(0).getFirstChild().getTextContent();
            map.put(MetaDataTypes.VERSION_NAME.name(), version);

            actualMetaData = "r:MessageID";
            String fileId = document.getElementsByTagName(actualMetaData).item(0).getFirstChild().getTextContent();
            map.put(MetaDataTypes.FILE_ID.name(), fileId);

            map.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
            map.put(MetaDataTypes.DELETED.name(), Boolean.toString(false));

        } catch (NullPointerException e) {
            log.error("Required attribute missing in xml file. The missing attribute is " + actualMetaData + "it must be repaired to continue");
            log.info("App ended process file " + multipartFile.getOriginalFilename());
            throw new MissingRequiredMetaDataException(actualMetaData);

        } catch (SAXParseException e){
            log.error("There is problem with XML structure");
            log.info("App ended process file " + multipartFile.getOriginalFilename());
            throw new ParsingFileContentSyntaxException(AllowedFileFormat.XML);

        } catch (Exception e){
            log.error(e.getMessage(),e.getCause());
        }

        return map;
    }
}
