package com.cz.uhk.bakalarka.app.controller;

import com.cz.uhk.bakalarka.app.dao.MessagePayloadMinIoDao;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.service.InputMessageValidationService;
import com.cz.uhk.bakalarka.app.service.JsonParserService;
import com.cz.uhk.bakalarka.app.service.MessageService;
import com.cz.uhk.bakalarka.app.service.XmlParserService;
import com.cz.uhk.bakalarka.app.support.IOSupport;
import com.cz.uhk.bakalarka.app.support.MetaDataSupport;
import io.minio.*;
import io.minio.messages.Item;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.http.HttpStatus;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import static org.junit.jupiter.api.Assertions.assertEquals;

@Log4j2
@RunWith(SpringRunner.class)
@AutoConfigureMockMvc
@TestPropertySource(
        locations = "classpath:application.properties")
@WebMvcTest(MessageController.class)
@ComponentScan(basePackages = "com.cz.uhk.bakalarka.app")
class AppApplicationTest {

    private MockMvc mockMvc;

    @Autowired
    private MessagePayloadMinIoDao messagePayloadDao;
    private MinioClient minioClient;
    private final String defaultBucket = MetaDataSupport.DEFAULT_BUCKET;


    @BeforeEach
    void initEach() {
        final String url = "http://127.0.0.1:9000";

        this.minioClient = MinioClient.builder().credentials("USER", "12345678").endpoint(url).build();
        InputMessageValidationService inputMessageValidationService = new InputMessageValidationService(messagePayloadDao, 500000);
        MessageService messageService = new MessageService(messagePayloadDao);
        JsonParserService jsonParserService = new JsonParserService();
        XmlParserService xmlParserService = new XmlParserService();
        this.mockMvc = MockMvcBuilders.standaloneSetup(new MessageController(messageService, inputMessageValidationService, jsonParserService, xmlParserService)).build();
    }

    @Test
    void create_json() throws Exception {
        log.info("Test for success JSON upload will start.");

        String content = IOSupport.readFile("./TestFiles/vzoroveJsonProGateway.json");

        prepairMinio();

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        log.info("Perform request will start.");
        //volání api a vykonání ukládání
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Get code, metadata, payload from minio and generating expected metadata.");
        String code = IOSupport.getCode(mvcResult.getResponse().getContentAsString());

        //vygenerování metaDat
        MetaData metaDataFromRequest = MetaDataSupport.getMetaDataWithCode(code, "application/json");

        //získání metadat z minio
        String metaDataFromMinio =  mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", code)).andReturn().getResponse().getContentAsString();


        //získání payloadu z minio
        String payloadFromMinio = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", code)).andReturn().getResponse().getContentAsString();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.ACCEPTED.value(), mvcResult.getResponse().getStatus());
        assertEquals("event:validation success\n" +
                "data:File was successfully validated\n" +
                "\n" +
                "event:success upload\n" +
                "data:File was successfully uploaded\n" +
                "\n" +
                "event:code\n" +
                "data:" + code + "\n\n", mvcResult.getResponse().getContentAsString());
        assertEquals(metaDataFromRequest.toString(), metaDataFromMinio);
        assertEquals(content, payloadFromMinio);

        log.info("Deleting created file.");
        messagePayloadDao.delete(code);
    }

    @Test
    void create_xml() throws Exception {
        log.info("Test for success XML upload will start.");
        String content = IOSupport.readFile("./TestFiles/vzoroveXmlProGateway.xml");

        prepairMinio();

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/xml", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Get code, metadata, payload from minio and generating expected metadata.");
        String code = IOSupport.getCode(mvcResult.getResponse().getContentAsString());

        //vygenerování metaDat
        MetaData metaDataFromRequest = MetaDataSupport.getMetaDataWithCode(code, "application/xml");

        //získání metadat z minio
        String metaDataFromMinio =  mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", code)).andReturn().getResponse().getContentAsString();

        //získání payloadu z minio
        String payloadFromMinio = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", code)).andReturn().getResponse().getContentAsString();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.ACCEPTED.value(), mvcResult.getResponse().getStatus());
        assertEquals("event:validation success\n" +
                "data:File was successfully validated\n" +
                "\n" +
                "event:success upload\n" +
                "data:File was successfully uploaded\n" +
                "\n" +
                "event:code\n" +
                "data:" + code + "\n\n", mvcResult.getResponse().getContentAsString());
        assertEquals(metaDataFromRequest.toString(), metaDataFromMinio);
        assertEquals(content, payloadFromMinio);

        log.info("Deleting created file.");
        messagePayloadDao.delete(code);
    }

    @Test
    void create_emptyFile() throws Exception {
        log.info("Test to success handle empty file will start in create request.");
        String content = IOSupport.readFile("./TestFiles/empty.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:File is empty
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_largeFile() throws Exception {
        log.info("Test to success handle large file will start in create request.");
        String content = IOSupport.readFile("./TestFiles/largeFile.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:The file is larger than is allowed
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_badSyntaxJson() throws Exception {
        log.info("Test to success handle file with bad JSON syntax in create request.");
        String content = IOSupport.readFile("./TestFiles/badSyntax.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:In the file is syntax error
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_missingAttributeJson() throws Exception {
        log.info("Test to success handle missingAttribute in JSON file in create request.");
        String content = IOSupport.readFile("./TestFiles/missingAttribute.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:Required attribute missing in the file. The missing attribute is sendingAccountID, it must be repaired to continue
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_badSyntaxXml() throws Exception {
        log.info("Test to success handle file with bad XML syntax in create request.");
        String content = IOSupport.readFile("./TestFiles/badSyntax.xml");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:In the file is syntax error
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_missingAttributeXml() throws Exception {
        log.info("Test to success handle missingAttribute in XML file  in create request.");
        String content = IOSupport.readFile("./TestFiles/missingAttribute.xml");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/xml", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:Required attribute missing in the file. The missing attribute is r:SendingAccountID, it must be repaired to continue
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_unsupportedFileFormat() throws Exception {
        log.info("Test to success handle unsupported file format in create request.");
        String content = IOSupport.readFile("./TestFiles/unsupportedFileFormat.txt");

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/xml", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:In the file is syntax error
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_nonExistentBucket() throws Exception {
        log.info("Test to success handle no existent bucket in create request.");
        String content = IOSupport.readFile("./TestFiles/vzoroveJsonProGateway.json");

        deleteBucket();

        InputStream payload = new ByteArrayInputStream(content.getBytes());
        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.ACCEPTED.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:validation success
                data:File was successfully validated
              
                event:upload unsuccessful
                data:There is error response from repository during uploading, file wasn´t upload. With file name originalFileName, try again later. Also it may be non-existent bucket
                
                """, mvcResult.getResponse().getContentAsString());

        log.info("Creating new bucket.");
        minioClient.makeBucket(MakeBucketArgs.builder().bucket(defaultBucket).build());
    }

    @Test
    void getByCode_nonExistentObject() throws Exception {
        log.info("Test to success handle no existent object in getByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + codeOfNonExistentObject + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getByCode_nonExistentBucket() throws Exception {
        log.info("Test to success handle no existent bucket in getByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        deleteBucket();
        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + codeOfNonExistentObject + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());

        log.info("Creating new bucket.");
        minioClient.makeBucket(MakeBucketArgs.builder().bucket(defaultBucket).build());
    }

    @Test
    void getByCode_badCodePattern() throws Exception {
        log.info("Test to success handle bad code pattern in getByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_nonExistentObject() throws Exception {
        log.info("Test to success handle no existent object in getDataByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + codeOfNonExistentObject + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_nonExistentBucket() throws Exception {
        log.info("Test to success handle no existent bucket in getDataByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        deleteBucket();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + codeOfNonExistentObject + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());

        log.info("Creating new bucket.");
        minioClient.makeBucket(MakeBucketArgs.builder().bucket(defaultBucket).build());
    }

    @Test
    void getDataByCode_badCodePattern() throws Exception {
        log.info("Test to success handle bad code pattern in getDataByCode request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete() throws Exception {
        log.info("Test to success delete request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.OK.value(), mvcResult.getResponse().getStatus());
        assertEquals("The file with code: " + codeOfNonExistentObject + " was deleted", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_badCodePattern() throws Exception {
        log.info("Test to success handle bad code pattern in delete request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_nonExistentObject() throws Exception {
        log.info("Test to success handle no existent object in delete request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        prepairMinio();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.OK.value(), mvcResult.getResponse().getStatus());
        assertEquals("The file with code: " + codeOfNonExistentObject + " was deleted", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_nonExistentBucket() throws Exception {
        log.info("Test to success handle no existent bucket in delete request.");
        String codeOfNonExistentObject = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";

        deleteBucket();

        //volání api a vykonání ukládání
        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", codeOfNonExistentObject)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is error response from repository. It may be non existent bucket", mvcResult.getResponse().getContentAsString());

        log.info("Creating new bucket.");
        minioClient.makeBucket(MakeBucketArgs.builder().bucket(defaultBucket).build());
    }


    private void prepairMinio() throws Exception {

        //vytvoří bucket v případě že neexistuje
        if (!minioClient.bucketExists(BucketExistsArgs.builder().bucket(defaultBucket).build())) {
            log.info("Creating new bucket.");
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(defaultBucket).build());
        } else {
            log.info("Deleting all object in " + defaultBucket +" bucket.");
            //vyprázdní se bucket
            Iterable<Result<Item>> listOfAllObjects = minioClient.listObjects(ListObjectsArgs.builder().bucket(defaultBucket).build());

            for (Result<Item> object : listOfAllObjects) {
                String name = object.get().objectName();
                messagePayloadDao.delete(name);
            }
        }
    }


    private void deleteBucket() throws Exception {
        //vytvoří bucket v případě že neexistuje
        if (minioClient.bucketExists(BucketExistsArgs.builder().bucket(defaultBucket).build())) {
            //vyprázdní se bucket
            log.info("Deleting all object in " + defaultBucket +" bucket.");
            Iterable<Result<Item>> listOfAllObjects = minioClient.listObjects(ListObjectsArgs.builder().bucket(defaultBucket).build());

            for (Result<Item> object : listOfAllObjects) {
                String name = object.get().objectName();
                messagePayloadDao.delete(name);
            }
            log.info("Deleting bucket.");
            minioClient.removeBucket(RemoveBucketArgs.builder().bucket(defaultBucket).build());
        }
    }
}


