package com.cz.uhk.bakalarka.app.controller;

import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import com.cz.uhk.bakalarka.app.service.InputMessageValidationService;
import com.cz.uhk.bakalarka.app.service.JsonParserService;
import com.cz.uhk.bakalarka.app.service.MessageService;
import com.cz.uhk.bakalarka.app.service.XmlParserService;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.*;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.http.HttpStatus;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.multipart.MultipartFile;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;

@Log4j2
@RunWith(SpringRunner.class)
@AutoConfigureMockMvc
@TestPropertySource(
        locations = "classpath:application.properties")
@WebMvcTest(MessageController.class)
@ComponentScan(basePackages = "com.cz.uhk.bakalarka.app")
@ExtendWith(MockitoExtension.class)
class MessageControllerTest {

    @MockBean
    private MessageService messageService;

    @MockBean
    private InputMessageValidationService inputMessageValidationServiceInf;

    @MockBean
    private JsonParserService jsonParserService;
    @MockBean
    private XmlParserService xmlParserService;

    private MockMvc mockMvc;

    @BeforeEach
    void setUp() {
         this.mockMvc = MockMvcBuilders.standaloneSetup(new MessageController(messageService, inputMessageValidationServiceInf,jsonParserService,xmlParserService)).build();
    }

    @Test
    void create() throws Exception {
        log.info("Test for success upload.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doReturn(new String[]{
                AllowedFileFormat.JSON.name(), "code"
        }).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.ACCEPTED.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:validation success
                data:File was successfully validated
                
                """, mvcResult.getResponse().getContentAsString());
    }


    //natvrdo vyhozená NullPointer na prvním místě kde to jde z důvodu, že perform nevezme null file. Testování zda nevybrání souboru vyvolá
    //daný http response status s message
    @Test
    void create_noFileSelected_catchingException() throws Exception {
        log.info("Test for success handle Exception, if no file is selected in create request");
        InputStream payload = new ByteArrayInputStream(new byte[]{
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doThrow(NullPointerException.class).when(inputMessageValidationServiceInf).validateCreate(any(MultipartFile.class));

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:Transfer problem
                data:Problem with file transfer to api
                
                """, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_catchingEmptyFileException() throws Exception {
        log.info("Test to success handle empty file exception in create request.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
                  });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doThrow(EmptyFileException.class).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:File is empty
                
                """ , mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_catchingLargeFileException() throws Exception {
        log.info("Test to success handle large file exception in create request.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doThrow(LargeFileException.class).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:The file is larger than is allowed
                
                """ , mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_catchingJsonMissingRequiredMetaDataException() throws Exception {
        log.info("Test to success handle missingAttribute exception in create request.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doReturn(new String[]{
                AllowedFileFormat.JSON.name(), "code"
        }).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        Mockito.doThrow(new MissingRequiredMetaDataException("sendingAccountID")).when(jsonParserService).parseToMap(multipartFile);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:Required attribute missing in the file. The missing attribute is sendingAccountID, it must be repaired to continue
               
                """ , mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_catchingXmlMissingRequiredMetaDataException() throws Exception {
        log.info("Test to success handle missingAttribute exception in create request.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);

        Mockito.doReturn(new String[]{
                AllowedFileFormat.XML.name(), "code"
        }).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        Mockito.doThrow(new MissingRequiredMetaDataException("sendingAccountID")).when(xmlParserService).parseToMap(multipartFile);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:Required attribute missing in the file. The missing attribute is sendingAccountID, it must be repaired to continue
                
                """ , mvcResult.getResponse().getContentAsString());
    }

    @Test
    void create_catchingUnsupportedFileFormatException() throws Exception {
        log.info("Test to success handle unsupported file format exception in create request.");
        InputStream payload = new ByteArrayInputStream(new byte[]{
        });

        MockMultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", "application/json", payload);


        Mockito.doThrow(new UnsupportedFileFormatException()).when(inputMessageValidationServiceInf).validateCreate(multipartFile);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.multipart("/api/create").file(multipartFile)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("""
                event:File exception
                data:This file format server cannot processed
                
                """ , mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getByCode() throws Exception{
        log.info("Test to success return metadata based on the info about code in getByCode request.");

        Map<String, String> metaDataMap = new HashMap<>();
        metaDataMap.put(MetaDataTypes.CODE.name(), "code");
        metaDataMap.put(MetaDataTypes.CONTENT_TYPE.name(), "application/json");
        metaDataMap.put(MetaDataTypes.FILE_ID.name(), "2333");
        metaDataMap.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.DELETED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.FILE_NAME.name(), "testFile.txt");
        metaDataMap.put(MetaDataTypes.VERSION_NAME.name(), "2222");
        metaDataMap.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), "36382");
        metaDataMap.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), "237336");

        MetaData metaData = new MetaData(metaDataMap);
        Mockito.doReturn(metaData).when(messageService).getByCode("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.OK.value(), mvcResult.getResponse().getStatus());
        assertEquals(metaData.toString(), mvcResult.getResponse().getContentAsString());

    }

    @Test
    void getByCode_catchingPayloadDaoErrorResponseException() throws Exception{
        log.info("Test to success handle payloadDaoErrorResponseException in getByCode request.");
        String code = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";
        Mockito.doThrow(PayloadDaoErrorResponseException.class).when(messageService).getByCode(code);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", code)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + code + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getByCode_catchingPayloadDaoException() throws Exception{
        log.info("Test to success handle payloadDaoException in getByCode request.");
        Mockito.doThrow(PayloadDaoException.class).when(messageService).getByCode("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is internal error, try again later", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getByCode_catchingBadCodeFormatException() throws Exception {
        log.info("Test to success handle bad code format exception in getByCode request.");
        Mockito.doThrow(BadCodeFormatException.class).when(inputMessageValidationServiceInf).validateCodeCorrectness("");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getByCode").param("code","")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode() throws Exception{
        log.info("Test to success return payload based on the info about code in getDataByCode request.");
        String payloadString = "Example";
        InputStream payload = new ByteArrayInputStream(payloadString.getBytes());

        Mockito.doReturn(payload).when(messageService).getDataByCode("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.OK.value(), mvcResult.getResponse().getStatus());
        assertEquals(payloadString, mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_catchingPayloadDaoErrorResponseException() throws Exception{
        log.info("Test to success handle payloadDaoErrorResponseException in getDataByCode request.");
        String code = "00a6d06f-6e3c-4e21-9cc4-8e229d29ecc9";
        Mockito.doThrow(PayloadDaoErrorResponseException.class).when(messageService).getDataByCode(code);

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", code)).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.NOT_FOUND.value(), mvcResult.getResponse().getStatus());
        assertEquals("File with code: " + code + " doesn´t exist or bucket doesn´t exist", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_catchingPayloadDaoException() throws Exception{
        log.info("Test to success handle payloadDaoException in getDataByCode request.");
        Mockito.doThrow(PayloadDaoException.class).when(messageService).getDataByCode("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is internal error, try again later", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_catchingException() throws Exception{
        log.info("Test to success handle exception in getDataByCode request.");
        Mockito.doThrow(IOException.class).when(Mockito.mock(InputStream.class));

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is internal IO error", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void getDataByCode_catchingBadCodeFormatException() throws Exception {
        log.info("Test to success handle bad code format exception in getDataByCode request.");
        Mockito.doThrow(BadCodeFormatException.class).when(inputMessageValidationServiceInf).validateCodeCorrectness("");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.get("/api/getDataByCode").param("code","")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete() throws Exception{
        log.info("Test to success delete file.");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.OK.value(), mvcResult.getResponse().getStatus());
        assertEquals("The file with code: code was deleted", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_catchingPayloadDaoErrorResponseException() throws Exception{
        log.info("Test to success handle payloadDaoErrorResponseException in delete request.");
        Mockito.doThrow(PayloadDaoErrorResponseException.class).when(messageService).delete("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is error response from repository. It may be non existent bucket", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_catchingPayloadDaoException() throws Exception{
        log.info("Test to success handle payloadDaoException in delete request.");
        Mockito.doThrow(PayloadDaoException.class).when(messageService).delete("code");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code", "code")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR.value(), mvcResult.getResponse().getStatus());
        assertEquals("There is internal error, try again later", mvcResult.getResponse().getContentAsString());
    }

    @Test
    void delete_catchingBadCodeFormatException() throws Exception {
        log.info("Test to success handle bad code format exception in delete request.");
        Mockito.doThrow(BadCodeFormatException.class).when(inputMessageValidationServiceInf).validateCodeCorrectness("");

        log.info("Perform request will start.");
        MvcResult mvcResult = mockMvc.perform(MockMvcRequestBuilders.delete("/api/delete").param("code","")).andReturn();

        log.info("Assert messages and HTTP status.");
        assertEquals(HttpStatus.BAD_REQUEST.value(), mvcResult.getResponse().getStatus());
        assertEquals("The selected code have bad pattern, try again", mvcResult.getResponse().getContentAsString());
    }
}