package com.cz.uhk.bakalarka.app.dao;

import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadErrorResponseException;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadException;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.model.enums.MetaDataTypes;
import com.cz.uhk.bakalarka.app.support.MetaDataSupport;
import io.minio.*;
import io.minio.errors.ErrorResponseException;
import io.minio.errors.InternalException;
import lombok.extern.log4j.Log4j2;
import okhttp3.Headers;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.*;
import org.mockito.junit.jupiter.MockitoExtension;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

@Log4j2
@ExtendWith(MockitoExtension.class)
class MessagePayloadMinIoDaoTest {

    private MinioClient minioClient;

    private MessagePayloadMinIoDao messagePayloadDao;

    @BeforeEach
    void setUp(){
        minioClient = mock(MinioClient.class);
        messagePayloadDao = new MessagePayloadMinIoDao(minioClient, MetaDataSupport.DEFAULT_BUCKET);
    }




    @Test
    void create() throws Exception{
        log.info("Test to success upload file into MinIo.");

        log.info("Generating testing metadata objects.");
        Map<String, String> userMetaDataFromResponse = new HashMap<>();

        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.CODE.name(), "code");
        userMetaDataFromResponse.put("Content-Type", "application/json");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.FILE_ID.name(), "2333");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.FILE_NAME.name(), "testFile.txt");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.VERSION_NAME.name(), "2222");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.DELETED.name(), Boolean.toString(false));
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.SENDING_ACCOUNT_ID.name(), "36382");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), "237336");

        Map<String, String> metaDataMap = new HashMap<>();

        metaDataMap.put(MetaDataTypes.CODE.name(), "code");
        metaDataMap.put(MetaDataTypes.CONTENT_TYPE.name(), "application/json");
        metaDataMap.put(MetaDataTypes.FILE_ID.name(), "2333");
        metaDataMap.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.DELETED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.FILE_NAME.name(), "testFile.txt");
        metaDataMap.put(MetaDataTypes.VERSION_NAME.name(), "2222");
        metaDataMap.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), "36382");
        metaDataMap.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), "237336");


        MetaData metaData = new MetaData(metaDataMap);

        userMetaDataFromResponse.put("Last-Modified", "Wed, 09 Apr 2008 23:55:38 GMT");
        StatObjectResponse statObjectResponse = new StatObjectResponse(Headers.of(userMetaDataFromResponse) , "b", "r","code");

        InputStream payload = new ByteArrayInputStream(new byte[] {
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });

        doReturn(statObjectResponse).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert expected and returned metadata.");
        assertEquals(metaData.toString(), messagePayloadDao.create(payload,metaData).toString());

    }
    @Test()
    void create_throwingMinioPayloadErrorResponseException() throws Exception{
        log.info("Test to success handle ErrorResponseException while uploading.");

        log.info("Creating testing metadata object with code");
        MetaData metaData = MetaDataSupport.getMetaDataWithCode();

        InputStream payload = new ByteArrayInputStream(new byte[] {
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });


        doThrow(ErrorResponseException.class).when(minioClient).putObject(Mockito.any(PutObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadErrorResponseException.class, () -> messagePayloadDao.create(payload,metaData));
    }

    @Test()
    void create_throwingMinioPayloadException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while uploading.");

        MetaData metaData = MetaDataSupport.getMetaDataWithCode();

        InputStream payload = new ByteArrayInputStream(new byte[] {
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });


        doThrow(InternalException.class).when(minioClient).putObject(Mockito.any(PutObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadException.class, () -> messagePayloadDao.create(payload,metaData));
    }

    @Test
    void delete_throwingMinioPayloadException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while deleting.");
        doThrow(InternalException.class).when(minioClient).removeObject(Mockito.any(RemoveObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadException.class, () -> messagePayloadDao.delete("code"));
    }

    @Test
    void delete_throwingMinioPayloadErrorResponseException() throws Exception{
        log.info("Test to success handle ErrorResponseException while deleting.");

        doThrow(ErrorResponseException.class).when(minioClient).removeObject(Mockito.any(RemoveObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadErrorResponseException.class, () -> messagePayloadDao.delete("code"));
    }

    @Test
    void getDataByCode() throws Exception{
        log.info("Test to success return payload from minio.");
        InputStream payload = new ByteArrayInputStream(new byte[] {
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });
        GetObjectResponse response = new GetObjectResponse(new Headers.Builder().build(), "h","r","o",payload);

        doReturn(response).when(minioClient).getObject(Mockito.any(GetObjectArgs.class));

        log.info("Assert expected and returned payload.");
        assertEquals(response, messagePayloadDao.getDataByCode("code"));
    }

    @Test
    void getDataByCode_throwingMinioPayloadErrorException() throws Exception{
        log.info("Test to success handle ErrorResponseException while returning payload.");
        doThrow(ErrorResponseException.class).when(minioClient).getObject(any(GetObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadErrorResponseException.class, ()-> messagePayloadDao.getDataByCode("code"));
    }

    @Test
    void getDataByCode_throwingMinioPayloadException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while returning payload.");
        doThrow(InternalException.class).when(minioClient).getObject(any(GetObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadException.class, ()-> messagePayloadDao.getDataByCode("code"));
    }

    @Test
    void getByCode() throws Exception{
        log.info("Test to success return metadata from minio.");

        log.info("Generating testing metadata objects.");
        Map<String, String> userMetaDataFromResponse = new HashMap<>();

        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.CODE.name(), "code");
        userMetaDataFromResponse.put("Content-Type", "application/json");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.FILE_ID.name(), "2333");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.FILE_NAME.name(), "testFile.txt");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.VERSION_NAME.name(), "2222");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.DELETED.name(), Boolean.toString(false));
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.SENDING_ACCOUNT_ID.name(), "36382");
        userMetaDataFromResponse.put("x-amz-meta-" + MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), "237336");

        Map<String, String> metaDataMap = new HashMap<>();

        metaDataMap.put(MetaDataTypes.CODE.name(), "code");
        metaDataMap.put(MetaDataTypes.CONTENT_TYPE.name(), "application/json");
        metaDataMap.put(MetaDataTypes.ARCHIVED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.DELETED.name(), Boolean.toString(false));
        metaDataMap.put(MetaDataTypes.FILE_ID.name(), "2333");
        metaDataMap.put(MetaDataTypes.FILE_NAME.name(), "testFile.txt");
        metaDataMap.put(MetaDataTypes.VERSION_NAME.name(), "2222");
        metaDataMap.put(MetaDataTypes.SENDING_ACCOUNT_ID.name(), "36382");
        metaDataMap.put(MetaDataTypes.RECEIVING_ACCOUNT_ID.name(), "237336");


        MetaData metaData = new MetaData(metaDataMap);

        userMetaDataFromResponse.put("Last-Modified", "Wed, 09 Apr 2008 23:55:38 GMT");
        StatObjectResponse statObjectResponse = new StatObjectResponse(Headers.of(userMetaDataFromResponse) , "b", "r","code");

        doReturn(statObjectResponse).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert expected and returned metadata.");
        assertEquals(metaData.getMetaDataMap(), messagePayloadDao.getByCode("code").getMetaDataMap());
    }

    @Test
    void getByCode_throwingMinioPayloadErrorResponseException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while returning metadata.");
        doThrow(ErrorResponseException.class).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadErrorResponseException.class, () -> messagePayloadDao.getByCode("code"));
    }

    @Test
    void getByCode_throwingMinioPayloadException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while returning payload.");
        doThrow(InternalException.class).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadException.class, () -> messagePayloadDao.getByCode("code"));
    }

    @Test
    void noExists() throws Exception{
        log.info("Test to success validation no exist object.");
        doThrow(ErrorResponseException.class).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert existence from dao with false");
        assertFalse(messagePayloadDao.exists("code"));
    }

    @Test
    void exists() throws Exception{
        log.info("Test to success validation exist object.");
        MetaData metaData = MetaDataSupport.getMetaDataWithCode();
        Map<String, String> metaDataMap = metaData.getMetaDataMap();
        metaDataMap.put("Last-Modified", "Wed, 09 Apr 2008 23:55:38 GMT");
        StatObjectResponse statObjectResponse = new StatObjectResponse(Headers.of(metaDataMap) , "b", "r","code");

        doReturn(statObjectResponse).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert existence from dao with true");
        assertTrue(messagePayloadDao.exists("code"));
    }

    @Test
    void exists_throwingMinioPayloadException() throws Exception{
        log.info("Test to success handle other exception, in this case representing by InternalException. Exception occurs while testing existence of object.");
        doThrow(InternalException.class).when(minioClient).statObject(any(StatObjectArgs.class));

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MinioPayloadException.class, ()-> messagePayloadDao.exists("code"));
    }
}