package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.dao.MessagePayloadMinIoDao;
import com.cz.uhk.bakalarka.app.service.enums.AllowedFileFormat;
import com.cz.uhk.bakalarka.app.service.exception.BadCodeFormatException;
import com.cz.uhk.bakalarka.app.service.exception.EmptyFileException;
import com.cz.uhk.bakalarka.app.service.exception.LargeFileException;
import com.cz.uhk.bakalarka.app.service.exception.UnsupportedFileFormatException;
import com.cz.uhk.bakalarka.app.support.IOSupport;
import com.cz.uhk.bakalarka.app.support.MetaDataSupport;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.mock.web.MockMultipartFile;

import java.io.InputStream;
import java.util.regex.Pattern;

import static org.junit.jupiter.api.Assertions.*;

@Log4j2
class InputMessageValidationServiceTest {

    private InputMessageValidationService inputMessageValidationService;

    @BeforeEach
    void setUp() {
        MessagePayloadMinIoDao messagePayloadDao = Mockito.mock(MessagePayloadMinIoDao.class);
        inputMessageValidationService = new InputMessageValidationService(messagePayloadDao, 500000);
    }

    @Test
    void validateCreate() throws Exception {
        log.info("Test to success validation size and format of the file. After success validation generate code.");

        String content = IOSupport.readFile("./TestFiles/vzoroveXmlProGateway.xml");

        MockMultipartFile file = new MockMultipartFile("file","originalName",MetaDataSupport.JSON_CONTENT_TYPE, content.getBytes());

        log.info("Generating expected response.");
        String[] expectedResponse = new String[]{
                AllowedFileFormat.JSON.name(), "^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$"
        };

        String[] response = inputMessageValidationService.validateCreate(file);
        Pattern regex = Pattern.compile(expectedResponse[1]);

        log.info("Assert expected response with response from the method.");
        assertEquals(expectedResponse[0] ,response[0]);
        assertTrue(regex.matcher(response[1]).matches());
        assertEquals(2,  response.length);
    }

    @Test
    void validateCreate_whenFileIsLargerThan500Kb() {
        log.info("Test to success throw large file exception.");
        String content = IOSupport.readFile("./TestFiles/largeFile.json");
        MockMultipartFile mockFile = new MockMultipartFile("file","originalName",MetaDataSupport.JSON_CONTENT_TYPE, content.getBytes());

        log.info("Assert thrown exception and expected exception.");
        assertThrows(LargeFileException.class, () -> inputMessageValidationService.validateCreate(mockFile));
    }

    @Test
    void validateCreate_whenFileIsEmpty() throws Exception {
        log.info("Test to success throw empty file exception");
        MockMultipartFile mockFile = new MockMultipartFile("file","originalName",MetaDataSupport.JSON_CONTENT_TYPE, InputStream.nullInputStream());

        log.info("Assert thrown exception and expected exception.");
        assertThrows(EmptyFileException.class, () -> inputMessageValidationService.validateCreate(mockFile));
    }

    @Test
    void validateCreate_whenFileFormatNotAllowed() {
        log.info("Test to success throw wrong file format exception");
        String content = IOSupport.readFile("./TestFiles/vzoroveJsonProGateway.json");

        MockMultipartFile mockFile = new MockMultipartFile("file","originalName","text", content.getBytes());

        log.info("Assert thrown exception and expected exception.");
        assertThrows(UnsupportedFileFormatException.class, () -> inputMessageValidationService.validateCreate(mockFile));
    }

    @Test
    void validateCode() throws Exception{
        log.info("Test to success validation code");
        String code = "98d4d1d3-1730-4196-b7ee-c708eeda2cec";
        assertTrue(inputMessageValidationService.validateCodeCorrectness(code));
    }

    @Test
    void validateCodeLength_patternDifferentThanUUID() {
        String code = "98d4d1d3-1730-4196-b7ee-c708eeda2ce";
        log.info("Assert thrown exception and expected exception.");
        assertThrows(BadCodeFormatException.class, () -> inputMessageValidationService.validateCodeCorrectness(code));
    }
}