package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.support.IOSupport;
import com.cz.uhk.bakalarka.app.service.exception.ParsingFileContentSyntaxException;
import com.cz.uhk.bakalarka.app.service.exception.MissingRequiredMetaDataException;
import com.cz.uhk.bakalarka.app.support.MetaDataSupport;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;


@Log4j2
@ExtendWith(MockitoExtension.class)
class ParserServiceTest {

    private JsonParserService jsonParserService;
    private XmlParserService xmlParserService;

    @BeforeEach
    void setUp() {
        jsonParserService = new JsonParserService();
        xmlParserService = new XmlParserService();
    }

    @Test
    void Json_parseToMap() throws Exception {
        log.info("Test to success JSON parse to map");
        String content = IOSupport.readFile("./TestFiles/vzoroveJsonProGateway.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", MetaDataSupport.JSON_CONTENT_TYPE, payload);

        log.info("Generating expected map");
        MetaData expectedMetaData = MetaDataSupport.getMetaDataWithoutCode(MetaDataSupport.JSON_CONTENT_TYPE);

        log.info("Assert expected map with generated map");
        assertEquals(expectedMetaData.getMetaDataMap(), jsonParserService.parseToMap(multipartFile));
    }

    @Test
    void Json_parseToMap_whenJsonRequiredAttributeMissing() throws Exception {
        log.info("Test to success handle missing attribute in JSON file");
        String content = IOSupport.readFile("./TestFiles/missingAttribute.json");
        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalName", MetaDataSupport.JSON_CONTENT_TYPE, payload);

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MissingRequiredMetaDataException.class, () -> jsonParserService.parseToMap(multipartFile));
    }

    @Test
    void Json_parseToMap_whenJsonFormattingIsBad() throws Exception {
        log.info("Test to success handle wrong formatting in JSON file");
        String content = IOSupport.readFile("./TestFiles/badSyntax.json");

        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalName", MetaDataSupport.JSON_CONTENT_TYPE, payload);

        log.info("Assert thrown exception and expected exception.");
        assertThrows(ParsingFileContentSyntaxException.class, () -> jsonParserService.parseToMap(multipartFile));
    }

    @Test
    void Xml_parseToMap() throws Exception {
        log.info("Test to success XML parse to map");
        String content = IOSupport.readFile("./TestFiles/vzoroveXmlProGateway.xml");
        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalFileName", MetaDataSupport.XML_CONTENT_TYPE, payload);

        log.info("Generating expected map");
        MetaData exptectedMetaData = MetaDataSupport.getMetaDataWithoutCode(MetaDataSupport.XML_CONTENT_TYPE);

        log.info("Assert expected map with generated map");
        assertEquals(exptectedMetaData.getMetaDataMap(), xmlParserService.parseToMap(multipartFile));
    }

    @Test
    void Xml_parseToMap_whenRequiredAttributeMissing() throws Exception {
        log.info("Test to success handle missing attribute in XML file");
        String content = IOSupport.readFile("./TestFiles/missingAttribute.xml");
        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalName", MetaDataSupport.XML_CONTENT_TYPE, payload);

        log.info("Assert thrown exception and expected exception.");
        assertThrows(MissingRequiredMetaDataException.class, () -> xmlParserService.parseToMap(multipartFile));
    }

    @Test
    void Xml_parseToMap_whenXmlFormattingIsBad() throws Exception {
        log.info("Test to success handle wrong formatting in XML file");
        String content = IOSupport.readFile("./TestFiles/badSyntax.xml");
        InputStream payload = new ByteArrayInputStream(content.getBytes());

        MultipartFile multipartFile = new MockMultipartFile("file", "originalName", MetaDataSupport.XML_CONTENT_TYPE, payload);

        log.info("Assert thrown exception and expected exception.");
        assertThrows(ParsingFileContentSyntaxException.class, () -> xmlParserService.parseToMap(multipartFile));
    }
}